﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Resources;
using System.Windows;
using System.Xml.Linq;

namespace Telerik.Windows.QuickStart
{
	public class ExamplesViewModel : NotifyPropertyChangedBase
	{
		private ExamplesCollection examples;
		private IExampleItem selectedExample;
		private string searchText;

		public string SearchText
		{
			get
			{
				return this.searchText;
			}
			set
			{
				if (this.searchText != value)
				{
					this.searchText = value;
					this.OnPropertyChanged("SearchText");

					this.Examples.ExpandedAll(!string.IsNullOrEmpty(this.searchText));
					this.Examples.ApplyFilter(this.searchText);
				}
			}
		}

		public ExamplesCollection Examples
		{
			get
			{
				if (this.examples == null)
				{
					this.examples = LoadExamples();
				}
				return this.examples;
			}
		}

		public IExampleItem SelectedExample
		{
			get
			{
				return this.selectedExample;
			}
			set
			{
				if (!value.IsExample && value.Parent == null)
				{
					value = value.CategoryFirstLookExample;
				}
				if (this.selectedExample != value)
				{
					this.selectedExample = value;
					if (this.selectedExample != null)
					{
						this.OnSelectedExampleChanged();
					}
					this.OnPropertyChanged("SelectedExample");
				}
			}
		}

		public event EventHandler SelectedExampleChanged = delegate { };

		protected virtual void OnSelectedExampleChanged()
		{
			this.SelectedExampleChanged(this, EventArgs.Empty);
		}

		public void SelectFirstVisibleExample()
		{
			var example = this.FindExample(this.Examples, (item) => item.IsExample && item.Visible);

			this.SearchText = string.Empty;

			this.SelectedExample = example;
		}

		public void FindExampleBypath(string path)
		{
			this.SelectedExample = this.FindExample(this.Examples, (item) => item.Path == path);
		}

		private IExampleItem FindExample(IEnumerable<IExampleItem> collection, Predicate<IExampleItem> condition)
		{
			foreach (IExampleItem item in collection)
			{
				if (condition(item))
				{
					return item;
				}
				else
				{
					IExampleItem child = this.FindExample(item.Children, condition);
					if (child != null)
					{
						return child;
					}
				}
			}
			return null;
		}

		public string BrowserTitle
		{
			get
			{
				return this.GetTitle(this.SelectedExample);
			}
		}

		private string GetTitle(IExampleItem item)
		{
			string titleDescription = string.Empty;

			if (item.IsHomepage)
			{
				titleDescription = string.Format(CultureInfo.InvariantCulture, "Highlights of RadControls {0}", item.Platform);
			}
			else
			{
				string exampleTitle = string.Empty;
				titleDescription = this.GetTitleText(item, ref exampleTitle);
				titleDescription = string.Format(CultureInfo.InvariantCulture, "Telerik Rad{0} {1} {2}", titleDescription, item.Platform, exampleTitle);
			}
			return titleDescription;
		}

		private string GetTitleText(IExampleItem item, ref string exampleTitle)
		{
			if (item.Parent != null)
			{
				exampleTitle = string.Format(" - {0}{1}", item.Text, exampleTitle);
				return this.GetTitleText(item.Parent, ref exampleTitle);
			}
			else
			{
				return item.Text;
			}
		}

		internal static ExamplesCollection LoadExamples()
		{
			string uri = BrowserHelper.PathToExampleXML;

			Uri examplesXml = new Uri(uri, UriKind.RelativeOrAbsolute);

			var resource = Application.GetResourceStream(examplesXml);
			if (resource == null)
			{
				throw new MissingManifestResourceException("Cannot find Examples.xml");
			}
			var items = XElement.Load(new StreamReader(resource.Stream));

			return ExampleItem.SelectAll(items.Elements());
		}
	}
}
